/*
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef CURLPP_EXCEPTION_HPP
#define CURLPP_EXCEPTION_HPP

#include <stdexcept>


namespace curlpp
{
#include <curl/curl.h>
  
  /*
   * This class is a parent to all curlpp's exceptions. This class takes
   * a const std::string & as argument.
   */
  class exception
  {
  public:
    // Copy constructor
    exception( const exception& other ) throw() : m_what( other.m_what )
    {}

    // Return the reason of the exception
    const char *what() const throw()
    {
      return m_what.c_str();
    }

    virtual ~exception()
    {}

  protected:
    exception( const std::string &what ) throw() : m_what( what )
    {}

    exception( const char *what ) throw() : m_what( what )
    {}

  private:
    // We can't instatiate an empty exception
    exception();

    std::string m_what;
  };

  /*
   * This class is a parent to all curlpp's runtime_errors. This class takes
   * a const std::string & as argument for it's parent: std::runtime_errors. This 
   * class is thrown when curlpp is encountering an error, but for runtime 
   * considerations, "unpredictable" by the library user.
   */
  class runtime_error : public curlpp::exception
  {
  public:
    runtime_error(const char *reason);
    runtime_error(const std::string &string);
  };

  /*
   * This class is a parent to all curlpp's logic_errors. This class takes
   * a const std::string & as argument for it's parent: std::logic_errors. This 
   * class is thrown when curlpp is encountering an error, but for logic 
   * considerations, "predictable" by the library user. Predictable means 
   * that the library user is missusing the library.
   */
  class logic_error : public curlpp::exception
  {
  public:
    logic_error(const char *reason);
    logic_error(const std::string &string);
  };

  /*
   * This is a class derived from curlpp::runtime_error, that takes a const 
   * char * and a CURLcode as arguments. This class is thrown when libcurl is
   * returning an error with a CURLcode, but for runtime considerations, 
   * "unpredictable" by the library user.
   */
  class libcurl_runtime_error : public curlpp::runtime_error
  {
  public:
    libcurl_runtime_error( const std::string &reason, curlpp::CURLcode code );
    libcurl_runtime_error( const char *reason, curlpp::CURLcode code );

    /* 
     * return the CURLcode that libcurl returned
     */
    curlpp::CURLcode what_code() const throw();

  private:
    curlpp::CURLcode m_code;
  };

  /*
   * This is a class derived from curlpp::logic_error, that takes a const 
   * char * and a CURLcode as arguments. This class is thrown when libcurl is
   * returning an error with a CURLcode,  but for logic considerations, 
   * "predictable" by the library user. Predictable means that the library 
   * user is missusing the library.
   */
  class libcurl_logic_error : public curlpp::logic_error
  {
  public:
    libcurl_logic_error(const std::string &reason, curlpp::CURLcode code);
    libcurl_logic_error(const char *reason, curlpp::CURLcode code);

    /* 
     * return the CURLcode that libcurl returned
     */
    curlpp::CURLcode what_code() const throw();

  private:
    curlpp::CURLcode m_code;
  };

  /**
   * if CURLcode is not equal to CURLE_OK, it throws a 
   * curlpp::libcurl_runtime_error with the reason and the code. It's used 
   * in inline fonction, because throwing an execption is heavy in binary 
   * code, something we don't want in inline fonctions.
   */
  void libcurl_runtime_assert( const std::string &reason, curlpp::CURLcode code );
  void libcurl_runtime_assert( const char *reason, curlpp::CURLcode code );

  /**
   * if CURLcode is not equal to CURLE_OK, it throws a 
   * curlpp::libcurl_logic_error with the reason and the code. It's used 
   * in inline fonction, because throwing an execption is heavy in binary 
   * code, something we don't want in inline fonctions.
   */
  void libcurl_logic_assert( const std::string &reason, curlpp::CURLcode code );
  void libcurl_logic_assert( const char *reason, curlpp::CURLcode code );

  /**
   * if is_okay is false, it throws a curlpp::runtime_error 
   * with the reason. It's used in inline fonction, because throwing 
   * an execption is heavy in binary code, something we don't want in 
   * an inline fonction.
   */
  void runtime_assert( const std::string &reason, bool is_okay );
  void runtime_assert( const char *reason, bool is_okay );

  /**
   * if is_true is false, it throws a curl::logic_error with 
   * the reason. It's used in inline fonction, because throwing 
   * an execption is heavy in binary code, something we don't 
   * want in an inline fonction.
   */
  void logic_assert( const std::string &reason, bool is_okay );
  void logic_assert( const char *reason, bool is_okay );

}
#endif
