/**
 *    Copyright (c) <2002-2003> <Jean-Philippe Barrette-LaPierre>
 *    
 *    Permission is hereby granted, free of charge, to any person obtaining
 *    a copy of this software and associated documentation files 
 *    (curlpp), to deal in the Software without restriction, 
 *    including without limitation the rights to use, copy, modify, merge,
 *    publish, distribute, sublicense, and/or sell copies of the Software,
 *    and to permit persons to whom the Software is furnished to do so, 
 *    subject to the following conditions:
 *    
 *    The above copyright notice and this permission notice shall be included
 *    in all copies or substantial portions of the Software.
 *    
 *    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *    OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 *    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY 
 *    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, 
 *    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef CURLPP_INFO_HPP
#define CURLPP_INFO_HPP

#include <string>

namespace curlpp {
   class easy;
   class curl;
   
   /**
    * This class keep internal information of the curl session.
    * This class is returned by a curlpp::easy::perform_info().
    */
   class info 
   {
      private:
         friend class curlpp::easy;
         
         info(curlpp::curl &handle);
         info();

      public:
         /**
          * Returns the last used effective URL.
          */
         std::string effective_url();

         /**
          * Returns the  last received HTTP code.
          */
         long http_code();

         /**
          * Returns the remote time of the retrieved document. If you get
          * -1, it can  be  because  of  many  reasons  (unknown, the
          *  server hides it or the server doesn't support the 
          *  command that tells document time etc) and the time 
          *  of the document is unknown.  Note  that  you  must 
          *  tell the server to collect this information before 
          *  the transfer is made, by using the curlpp::easy::file_time()
          *  function
          */
         long file_time();

         /** 
          * Returns the total transaction time in seconds for the transfer. 
          * This time does not include the  connect time,  so if you want 
          * the complete operation time, you should add the 
          * curlpp::curl_info::connect_time().
          */
         double total_time();

         /** 
          * Return the time, in seconds, it took from  the start until the 
          * name resolving was completed.
          */
         double namelookup_time();

         /**
          * Returns the time, in seconds, it took from the 
          * start until the connect to the  remote  host  (or proxy) 
          * was completed. 
          */
         double connect_time();

         /**
          * Returns the time, in seconds,  it  took  from the start 
          * until the first byte is just about to be transfered. This 
          * includes curlpp::curl_info::pretransfer_time() and  also 
          * the time the server needs to calculate the result.
          */
         double pretransfer_time();

         /**
          * Returns the  total time,  in  seconds,  it  took  for all 
          * redirection steps include name lookup,  connect,  pretransfer
          * and transfer before final transaction was started. 
          * curlpp::curl_info::redirect_time() contains the complete  
          * execution  time for multiple redirections.  
          */
         double redirect_time();

         /**
          * Returns the total number of  redirections that were 
          * actually followed.
          */
         long redirect_count();

         /**
          * Returns the  total amount of bytes that were uploaded.
          */
         double size_upload();

         /**
          * Returns the total amount of bytes that were downloaded.  
          * The  amount is  only for the latest transfer.
          */
         double size_download();

         /**
          * Returns the  average download speed that curl measured for
          * the complete download.
          */
         double speed_download();

         /**
          * Return the  average upload  speed  that curl measured for the 
          * complete upload.
          */
         double speed_upload();

         /**
          * Returns the total size of all the headers received.
          */
         long header_size();

         /**
          * Returns the total size of the issued requests. This is so  
          * far  only  for HTTP requests. Note that this may be more 
          * than one request if FOLLOWLOCATION is true.
          */
         long request_size();

         /**
          * Returns the result  of the  certification verification that 
          * was requested tent-type  of  the  downloaded object. This is 
          * the value read from the Content-Type:  field.  If  you
          * get  NULL,  it means that the server didn't send a 
          * valid Content-Type header  or  that  the  protocol 
          * used doesn't support this.  
          */
         long ssl_verifyresult();

      private:
         std::string m_effective_url;
         long m_http_code;
         long m_file_time;
         double m_total_time;
         double m_namelookup_time;
         double m_connect_time;
         double m_pretransfer_time;
         double m_starttransfer_time;
         double m_redirect_time;
         long m_redirect_count;
         double m_size_upload;
         double m_size_download;
         double m_speed_upload;
         double m_speed_download;
         long m_header_size;
         long m_request_size;
         long m_ssl_verifyresult;
   };
};

#endif
